@@ -27,9 +27,11 @@
 #include "global.h"
 
 #include "error.h"
+#include "key.h"
 #include "verificationresult.h" // for Signature::Notation
 
 #include <memory>
+#include <string>
 #include <vector>
 #include <utility>
 #include <iosfwd>
@@ -37,7 +39,6 @@
 namespace GpgME
 {
 
-class Key;
 class Data;
 class TrustItem;
 class ProgressProvider;
@@ -189,18 +190,35 @@ public:
         ExportSecret = 16,
         ExportRaw = 32,
         ExportPKCS12 = 64,
-        ExportNoUID = 128,
+        ExportNoUID = 128, // obsolete; has no effect
         ExportSSH = 256,
+        ExportSecretSubkey = 512,
     };
 
     GpgME::Error exportPublicKeys(const char *pattern, Data &keyData);
-    GpgME::Error exportPublicKeys(const char *pattern, Data &keyData, unsigned int flags);
+    GpgME::Error exportPublicKeys(const char *pattern, Data &keyData, unsigned int mode);
     GpgME::Error exportPublicKeys(const char *pattern[], Data &keyData);
-    GpgME::Error exportPublicKeys(const char *pattern[], Data &keyData, unsigned int export_mode);
+    GpgME::Error exportPublicKeys(const char *pattern[], Data &keyData, unsigned int mode);
     GpgME::Error startPublicKeyExport(const char *pattern, Data &keyData);
-    GpgME::Error startPublicKeyExport(const char *pattern, Data &keyData, unsigned int flags);
+    GpgME::Error startPublicKeyExport(const char *pattern, Data &keyData, unsigned int mode);
     GpgME::Error startPublicKeyExport(const char *pattern[], Data &keyData);
-    GpgME::Error startPublicKeyExport(const char *pattern[], Data &keyData, unsigned int export_mode);
+    GpgME::Error startPublicKeyExport(const char *pattern[], Data &keyData, unsigned int mode);
+
+    GpgME::Error exportSecretKeys(const char *pattern, Data &keyData, unsigned int mode = ExportSecret);
+    GpgME::Error exportSecretKeys(const char *pattern[], Data &keyData, unsigned int mode = ExportSecret);
+    GpgME::Error startSecretKeyExport(const char *pattern, Data &keyData, unsigned int mode = ExportSecret);
+    GpgME::Error startSecretKeyExport(const char *pattern[], Data &keyData, unsigned int mode = ExportSecret);
+
+    GpgME::Error exportSecretSubkeys(const char *pattern, Data &keyData, unsigned int mode = ExportSecretSubkey);
+    GpgME::Error exportSecretSubkeys(const char *pattern[], Data &keyData, unsigned int mode = ExportSecretSubkey);
+    GpgME::Error startSecretSubkeyExport(const char *pattern, Data &keyData, unsigned int mode = ExportSecretSubkey);
+    GpgME::Error startSecretSubkeyExport(const char *pattern[], Data &keyData, unsigned int mode = ExportSecretSubkey);
+
+    // generic export functions; prefer using the specific public/secret key export functions
+    GpgME::Error exportKeys(const char *pattern, Data &keyData, unsigned int mode = ExportDefault);
+    GpgME::Error exportKeys(const char *pattern[], Data &keyData, unsigned int mode = ExportDefault);
+    GpgME::Error startKeyExport(const char *pattern, Data &keyData, unsigned int mode = ExportDefault);
+    GpgME::Error startKeyExport(const char *pattern[], Data &keyData, unsigned int mode = ExportDefault);
 
     //
     // Key Import
@@ -208,8 +226,10 @@ public:
 
     ImportResult importKeys(const Data &data);
     ImportResult importKeys(const std::vector<Key> &keys);
+    ImportResult importKeys(const std::vector<std::string> &keyIds);
     GpgME::Error startKeyImport(const Data &data);
     GpgME::Error startKeyImport(const std::vector<Key> &keys);
+    GpgME::Error startKeyImport(const std::vector<std::string> &keyIds);
     ImportResult importResult() const;
 
     //
@@ -273,6 +293,23 @@ public:
                             unsigned long expires = 0,
                             unsigned int flags = 0);
 
+    enum SetExpireFlags {
+        SetExpireDefault = 0,
+        SetExpireAllSubkeys = 1
+    };
+
+    Error setExpire(const Key &k, unsigned long expires,
+                    const std::vector<Subkey> &subkeys = std::vector<Subkey>(),
+                    const SetExpireFlags flags = SetExpireDefault);
+    Error startSetExpire(const Key &k, unsigned long expires,
+                         const std::vector<Subkey> &subkeys = std::vector<Subkey>(),
+                         const SetExpireFlags flags = SetExpireDefault);
+
+    Error revokeSignature(const Key &key, const Key &signingKey,
+                          const std::vector<UserID> &userIds = std::vector<UserID>());
+    Error startRevokeSignature(const Key &key, const Key &signingKey,
+                               const std::vector<UserID> &userIds = std::vector<UserID>());
+
     // using TofuInfo::Policy
     Error setTofuPolicy(const Key &k, unsigned int policy);
     Error setTofuPolicyStart(const Key &k, unsigned int policy);
@@ -472,6 +509,7 @@ public:
     GpgME::Error wait();
     GpgME::Error lastError() const;
     GpgME::Error cancelPendingOperation();
+    GpgME::Error cancelPendingOperationImmediately();
 
     class Private;
     const Private *impl() const
