@@ -24,6 +24,7 @@
 #define __GPGMEPP_DATA_H__
 
 #include "global.h"
+#include "key.h"
 
 #include <sys/types.h> // for size_t, off_t
 #include <cstdio> // FILE
@@ -78,7 +79,10 @@ public:
         BinaryEncoding,
         Base64Encoding,
         ArmorEncoding,
-        MimeEncoding
+        MimeEncoding,
+        UrlEncoding,
+        UrlEscEncoding,
+        Url0Encoding,
     };
     Encoding encoding() const;
     Error setEncoding(Encoding encoding);
@@ -106,6 +110,16 @@ public:
     ssize_t write(const void *buffer, size_t length);
     off_t seek(off_t offset, int whence);
 
+    /* Convenience function to do a seek (0, SEEK_SET).  */
+    Error rewind();
+
+    /** Try to parse the data to a key object using the
+     * Protocol proto. Returns an empty list on error.*/
+    std::vector<Key> toKeys(const Protocol proto = Protocol::OpenPGP) const;
+
+    /** Return a copy of the data as std::string. Sets seek pos to 0 */
+    std::string toString();
+
     class Private;
     Private *impl()
     {
