@@ -66,7 +66,8 @@ enum KeyListMode {
     Validate = 0x10,
     Ephemeral = 0x20,
     WithTofu = 0x40,
-    WithKeygrip = 0x80
+    WithKeygrip = 0x80,
+    WithSecret = 0x100
 };
 
 enum SignatureMode { NormalSignatureMode, Detached, Clearsigned };
@@ -90,7 +91,8 @@ GPGMEPP_EXPORT EngineInfo engineInfo(Engine engine);
 homedir, sysconfdir, bindir, libexecdir, libdir,
 datadir, localedir, agent-socket, agent-ssh-socket,
 dirmngr-socket, uiserver-socket, gpgconf-name, gpg-name,
-gpgsm-name, g13-name
+gpgsm-name, g13-name, keyboxd-name, agent-name, scdaemon-name,
+dirmngr-name, pinentry-name, socketdir.
 
 This may be extended in the future.
 */
