@@ -35,6 +35,9 @@
 #include <cstring>
 
 #include <string.h>
+#include <strings.h>
+#include <istream>
+#include <iterator>
 
 class GpgME::ImportResult::Private
 {
@@ -45,17 +48,19 @@ public:
         // We just need to handle the pointers in the structs:
         for (gpgme_import_status_t is = r.imports ; is ; is = is->next) {
             gpgme_import_status_t copy = new _gpgme_import_status(*is);
-            copy->fpr = strdup(is->fpr);
-            copy->next = 0;
+            if (is->fpr) {
+                copy->fpr = strdup(is->fpr);
+            }
+            copy->next = nullptr;
             imports.push_back(copy);
         }
-        res.imports = 0;
+        res.imports = nullptr;
     }
     ~Private()
     {
         for (std::vector<gpgme_import_status_t>::iterator it = imports.begin() ; it != imports.end() ; ++it) {
             std::free((*it)->fpr);
-            delete *it; *it = 0;
+            delete *it; *it = nullptr;
         }
     }
 
@@ -192,7 +197,7 @@ bool GpgME::Import::isNull() const
 
 const char *GpgME::Import::fingerprint() const
 {
-    return isNull() ? 0 : d->imports[idx]->fpr ;
+    return isNull() ? nullptr : d->imports[idx]->fpr ;
 }
 
 GpgME::Error GpgME::Import::error() const
@@ -224,3 +229,42 @@ GpgME::Import::Status GpgME::Import::status() const
     }
     return static_cast<Status>(result);
 }
+
+std::ostream &GpgME::operator<<(std::ostream &os,
+                                const GpgME::ImportResult &result)
+{
+    os << "GpgME::ImportResult(";
+    if (!result.isNull()) {
+        os << "\n considered:          " << result.numConsidered()
+           << "\n without UID:         " << result.numKeysWithoutUserID()
+           << "\n imported:            " << result.numImported()
+           << "\n RSA Imported:        " << result.numRSAImported()
+           << "\n unchanged:           " << result.numUnchanged()
+           << "\n newUserIDs:          " << result.newUserIDs()
+           << "\n newSubkeys:          " << result.newSubkeys()
+           << "\n newSignatures:       " << result.newSignatures()
+           << "\n newRevocations:      " << result.newRevocations()
+           << "\n numSecretKeysConsidered: " << result.numSecretKeysConsidered()
+           << "\n numSecretKeysImported:   " << result.numSecretKeysImported()
+           << "\n numSecretKeysUnchanged:  " << result.numSecretKeysUnchanged()
+           << "\n"
+           << "\n notImported:         " << result.notImported()
+           << "\n numV3KeysSkipped:    " << result.numV3KeysSkipped()
+           << "\n imports:\n";
+        const std::vector<Import> imp = result.imports();
+        std::copy(imp.begin(), imp.end(),
+                  std::ostream_iterator<Import>(os, "\n"));
+    }
+    return os << ')';
+}
+
+std::ostream &GpgME::operator<<(std::ostream &os, const GpgME::Import &imp)
+{
+    os << "GpgME::Import(";
+    if (!imp.isNull()) {
+        os << "\n fpr:       " << (imp.fingerprint() ? imp.fingerprint() : "null")
+           << "\n status:    " << imp.status()
+           << "\n err:       " << imp.error();
+    }
+    return os << ')';
+}
