@@ -42,7 +42,7 @@
 #include "qgpgmelistallkeysjob.h"
 #include "qgpgmedecryptjob.h"
 #include "qgpgmedecryptverifyjob.h"
-#include "qgpgmerefreshsmimekeysjob.h"
+#include "qgpgmerefreshkeysjob.h"
 #include "qgpgmedeletejob.h"
 #include "qgpgmedownloadjob.h"
 #include "qgpgmesignencryptjob.h"
@@ -65,8 +65,6 @@
 #include "qgpgmetofupolicyjob.h"
 #include "qgpgmequickjob.h"
 #include "qgpgmereceivekeysjob.h"
-#include "qgpgmerevokekeyjob.h"
-#include "qgpgmesetprimaryuseridjob.h"
 
 namespace
 {
@@ -284,11 +282,12 @@ public:
 
     QGpgME::RefreshKeysJob *refreshKeysJob() const Q_DECL_OVERRIDE
     {
-        if (mProtocol != GpgME::CMS) {
+        if (mProtocol != GpgME::CMS) { // fixme: add support for gpg, too
             return nullptr;
         }
 
-        return new QGpgME::QGpgMERefreshSMIMEKeysJob;
+        // this operation is not supported by gpgme, so we have to call gpgsm ourselves:
+        return new QGpgME::QGpgMERefreshKeysJob();
     }
 
     QGpgME::DownloadJob *downloadJob(bool armor) const Q_DECL_OVERRIDE
@@ -422,7 +421,7 @@ public:
         if (!context) {
             return nullptr;
         }
-        context->setKeyListMode(GpgME::Locate | GpgME::Signatures | GpgME::Validate);
+        context->setKeyListMode(GpgME::Extern | GpgME::Local | GpgME::Signatures | GpgME::Validate);
         return new QGpgME::QGpgMEKeyListJob(context);
     }
 
@@ -482,30 +481,6 @@ public:
         }
         return new QGpgME::QGpgMEQuickJob(context);
     }
-
-    QGpgME::RevokeKeyJob *revokeKeyJob() const Q_DECL_OVERRIDE
-    {
-        if (mProtocol != GpgME::OpenPGP) {
-            return nullptr;
-        }
-        GpgME::Context *context = GpgME::Context::createForProtocol(mProtocol);
-        if (!context) {
-            return nullptr;
-        }
-        return new QGpgME::QGpgMERevokeKeyJob(context);
-    }
-
-    QGpgME::SetPrimaryUserIDJob *setPrimaryUserIDJob() const override
-    {
-        if (mProtocol != GpgME::OpenPGP) {
-            return nullptr;
-        }
-        GpgME::Context *context = GpgME::Context::createForProtocol(mProtocol);
-        if (!context) {
-            return nullptr;
-        }
-        return new QGpgME::QGpgMESetPrimaryUserIDJob{context};
-    }
 };
 
 }
