@@ -3,7 +3,10 @@
 
     This file is part of qgpgme, the Qt API binding for gpgme
     Copyright (c) 2008 Klarälvdalens Datakonsult AB
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
+    Copyright (c) 2021 g10 Code GmbH
+    Software engineering by Ingo Klöcker <dev@ingo-kloecker.de>
 
     QGpgME is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -31,21 +34,17 @@
     your version.
 */
 
+#ifdef HAVE_CONFIG_H
+ #include "config.h"
+#endif
+
 #include "qgpgmechangeexpiryjob.h"
 
-#include "dataprovider.h"
-
 #include "context.h"
-#include "data.h"
-#include "gpgsetexpirytimeeditinteractor.h"
 #include "key.h"
 
 #include <QDateTime>
 
-#include <cassert>
-#include <memory>
-#include <string>
-
 using namespace QGpgME;
 using namespace GpgME;
 
@@ -57,24 +56,45 @@ QGpgMEChangeExpiryJob::QGpgMEChangeExpiryJob(Context *context)
 
 QGpgMEChangeExpiryJob::~QGpgMEChangeExpiryJob() {}
 
-static QGpgMEChangeExpiryJob::result_type change_expiry(Context *ctx, const Key &key, const QDateTime &expiry)
+static QGpgMEChangeExpiryJob::result_type change_expiry(Context *ctx, const Key &key, const QDateTime &expiry,
+    const std::vector<Subkey> &subkeys, ChangeExpiryJob::Options options)
 {
-    EditInteractor *ei = expiry.isValid()
-       ? new GpgSetExpiryTimeEditInteractor(expiry.date().toString(Qt::ISODate).toStdString())
-       : new GpgSetExpiryTimeEditInteractor();
+    // convert expiry to "seconds from now"; use 1 second from now if expiry is before the current datetime
+    const unsigned long expires = expiry.isValid()
+       ? std::max<qint64>(QDateTime::currentDateTime().secsTo(expiry), 1)
+       : 0;
 
-    QGpgME::QByteArrayDataProvider dp;
-    Data data(&dp);
-    assert(!data.isNull());
-    const Error err = ctx->edit(key, std::unique_ptr<EditInteractor> (ei), data);
-    Error ae;
-    const QString log = _detail::audit_log_as_html(ctx, ae);
-    return std::make_tuple(err, log, ae);
+    // updating the expiration date of the primary key and the subkeys needs to be done in two steps
+    // because --quick-set-expire does not support updating the expiration date of both at the same time
+
+    if (subkeys.empty() || (options & ChangeExpiryJob::UpdatePrimaryKey)) {
+        // update the expiration date of the primary key
+        auto err = ctx->setExpire(key, expires);
+        if (err) {
+            return std::make_tuple(err, QString(), Error());
+        }
+    }
+
+    GpgME::Error err;
+    if (!subkeys.empty()) {
+        // update the expiration date of the specified subkeys
+        err = ctx->setExpire(key, expires, subkeys);
+    } else if (options & ChangeExpiryJob::UpdateAllSubkeys) {
+        // update the expiration date of all subkeys
+        err = ctx->setExpire(key, expires, {}, Context::SetExpireAllSubkeys);
+    }
+    return std::make_tuple(err, QString(), Error());
 }
 
 Error QGpgMEChangeExpiryJob::start(const Key &key, const QDateTime &expiry)
 {
-    run(std::bind(&change_expiry, std::placeholders::_1, key, expiry));
+    return start(key, expiry, std::vector<Subkey>());
+}
+
+Error QGpgMEChangeExpiryJob::start(const Key &key, const QDateTime &expiry, const std::vector<Subkey> &subkeys)
+{
+    run(std::bind(&change_expiry, std::placeholders::_1, key, expiry, subkeys, options()));
     return Error();
 }
+
 #include "qgpgmechangeexpiryjob.moc"
