@@ -54,7 +54,7 @@ _gpgme_assuan_log_cb (assuan_context_t ctx, void *hook,
   if (msg == NULL)
     return 1;
 
-  _gpgme_debug (DEBUG_ASSUAN, -1, NULL, NULL, NULL, "%s", msg);
+  _gpgme_debug (NULL, DEBUG_ASSUAN, -1, NULL, NULL, NULL, "%s", msg);
   return 0;
 }
 
@@ -234,7 +234,7 @@ my_spawn (assuan_context_t ctx, pid_t *r_pid, const char *name,
                   break;
                 }
               logger_fd = strtol (argv[loc], &tail, 10);
-              if (tail == argv[loc] || logger_fd <= 0)
+              if (tail == argv[loc] || logger_fd < 0)
                 {
                   err = GPG_ERR_INV_ARG;
                   break;
