@@ -258,11 +258,7 @@ gpgme_get_engine_info (gpgme_engine_info_t *info)
           if (!err && version && engine_minimal_version
               && !_gpgme_compare_versions (version, engine_minimal_version))
             {
-#if GPG_ERROR_VERSION_NUMBER < 0x011900 /* 1.25 */
-              err = gpg_error (GPG_ERR_NO_ENGINE);
-#else
               err = gpg_error (GPG_ERR_ENGINE_TOO_OLD);
-#endif
             }
 
           /* Now set the dummy version for pseudo engines.  */
@@ -824,6 +820,20 @@ _gpgme_engine_op_keysign (engine_t engine, gpgme_key_t key, const char *userid,
 }
 
 
+gpgme_error_t
+_gpgme_engine_op_revsig (engine_t engine, gpgme_key_t key, gpgme_key_t signing_key,
+                         const char *userid, unsigned int flags)
+{
+  if (!engine)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  if (!engine->ops->revsig)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  return (*engine->ops->revsig) (engine->engine, key, signing_key, userid, flags);
+}
+
+
 gpgme_error_t
 _gpgme_engine_op_tofu_policy (engine_t engine,
                               gpgme_key_t key,  gpgme_tofu_policy_t policy)
@@ -840,7 +850,8 @@ _gpgme_engine_op_tofu_policy (engine_t engine,
 
 gpgme_error_t
 _gpgme_engine_op_import (engine_t engine, gpgme_data_t keydata,
-                         gpgme_key_t *keyarray)
+                         gpgme_key_t *keyarray, const char *keyids[],
+                         const char *import_filter, const char *key_origin)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -848,7 +859,8 @@ _gpgme_engine_op_import (engine_t engine, gpgme_data_t keydata,
   if (!engine->ops->import)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->import) (engine->engine, keydata, keyarray);
+  return (*engine->ops->import) (engine->engine, keydata, keyarray, keyids,
+                                 import_filter, key_origin);
 }
 
 
@@ -917,13 +929,12 @@ _gpgme_engine_op_sign (engine_t engine, gpgme_data_t in, gpgme_data_t out,
 gpgme_error_t
 _gpgme_engine_op_trustlist (engine_t engine, const char *pattern)
 {
+  (void)pattern;
+
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
 
-  if (!engine->ops->trustlist)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->trustlist) (engine->engine, pattern);
+  return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 }
 
 
@@ -1129,3 +1140,17 @@ _gpgme_engine_op_spawn (engine_t engine,
   return (*engine->ops->opspawn) (engine->engine, file, argv,
                                   datain, dataout, dataerr, flags);
 }
+
+gpgme_error_t
+_gpgme_engine_op_setexpire (engine_t engine, gpgme_key_t key,
+                            unsigned long expires, const char *subfprs,
+                            unsigned int reserved)
+{
+  if (!engine)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  if (!engine->ops->setexpire)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  return (*engine->ops->setexpire) (engine->engine, key, expires, subfprs, reserved);
+}
