@@ -1,23 +1,23 @@
 /* t-import.c - Regression test.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2003, 2004 g10 Code GmbH
-
-   This file is part of GPGME.
- 
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-   
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-   
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2003, 2004 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 /* We need to include config.h so that we know whether we are building
    with large file system (LFS) support. */
@@ -36,9 +36,9 @@
 
 
 void
-check_result (gpgme_import_result_t result, char *fpr, int secret)
+check_result (gpgme_import_result_t result, const char *fpr, int secret)
 {
-  if (result->considered != 1)
+  if (result->considered != 1 && (secret && result->considered != 3))
     {
       fprintf (stderr, "Unexpected number of considered keys %i\n",
 	       result->considered);
@@ -63,7 +63,7 @@ check_result (gpgme_import_result_t result, char *fpr, int secret)
 	       result->imported_rsa);
       exit (1);
     }
-  if ((secret && result->unchanged != 0)
+  if ((secret && (result->unchanged != 0 && result->unchanged != 1))
       || (!secret && ((result->imported == 0 && result->unchanged != 1)
 		      || (result->imported == 1 && result->unchanged != 0))))
     {
@@ -101,14 +101,15 @@ check_result (gpgme_import_result_t result, char *fpr, int secret)
 	       result->new_revocations);
       exit (1);
     }
-  if ((secret && result->secret_read != 1)
+  if ((secret && result->secret_read != 1 && result->secret_read != 3)
       || (!secret && result->secret_read != 0))
     {
       fprintf (stderr, "Unexpected number of secret keys read %i\n",
 	       result->secret_read);
       exit (1);
     }
-  if ((secret && result->secret_imported != 0 && result->secret_imported != 1)
+  if ((secret && result->secret_imported != 0 && result->secret_imported != 1
+       && result->secret_imported != 2)
       || (!secret && result->secret_imported != 0))
     {
       fprintf (stderr, "Unexpected number of secret keys imported %i\n",
@@ -116,7 +117,8 @@ check_result (gpgme_import_result_t result, char *fpr, int secret)
       exit (1);
     }
   if ((secret
-       && ((result->secret_imported == 0 && result->secret_unchanged != 1)
+       && ((result->secret_imported == 0 && result->secret_unchanged != 1
+	    && result->secret_unchanged != 2)
 	   || (result->secret_imported == 1 && result->secret_unchanged != 0)))
       || (!secret && result->secret_unchanged != 0))
     {
@@ -147,16 +149,17 @@ check_result (gpgme_import_result_t result, char *fpr, int secret)
 	  exit (1);
 	}
     }
-  if (strcmp (fpr, result->imports->fpr))
+  if (!result->imports->fpr || strcmp (fpr, result->imports->fpr))
     {
       fprintf (stderr, "Unexpected fingerprint %s\n",
-	       result->imports->fpr);
+	       result->imports->fpr ? result->imports->fpr : "null");
       exit (1);
     }
-  if (result->imports->next && strcmp (fpr, result->imports->next->fpr))
+  if (result->imports->next
+  && (!result->imports->next->fpr || strcmp (fpr, result->imports->next->fpr)))
     {
       fprintf (stderr, "Unexpected fingerprint on second status %s\n",
-	       result->imports->next->fpr);
+	       result->imports->next->fpr ? result->imports->next->fpr : "null");
       exit (1);
     }
   if (result->imports->result != 0)
@@ -165,6 +168,7 @@ check_result (gpgme_import_result_t result, char *fpr, int secret)
 	       gpgme_strerror (result->imports->result));
       exit (1);
     }
+#if 0
   if (secret)
     {
       if (result->secret_imported == 0)
@@ -200,25 +204,34 @@ check_result (gpgme_import_result_t result, char *fpr, int secret)
 	  exit (1);
 	}
     }
+#endif
 }
 
 
-int 
+int
 main (int argc, char *argv[])
 {
   gpgme_ctx_t ctx;
   gpgme_error_t err;
   gpgme_data_t in;
   gpgme_import_result_t result;
-  const char *pubkey_1_asc = make_filename ("pubkey-1.asc");
-  const char *seckey_1_asc = make_filename ("seckey-1.asc");
+  gpgme_key_t key;
+  char *pubkey_1_asc = make_filename ("pubkey-1.asc");
+  char *seckey_1_asc = make_filename ("seckey-1.asc");
+
+  (void)argc;
+  (void)argv;
 
   init_gpgme (GPGME_PROTOCOL_OpenPGP);
 
   err = gpgme_new (&ctx);
   fail_if_err (err);
 
+  err = gpgme_set_ctx_flag (ctx, "key-origin", "wkd,https://openpgpkey.gnupg.org");
+  fail_if_err (err);
+
   err = gpgme_data_new_from_file (&in, pubkey_1_asc, 1);
+  free (pubkey_1_asc);
   fail_if_err (err);
 
   err = gpgme_op_import (ctx, in);
@@ -228,6 +241,7 @@ main (int argc, char *argv[])
   gpgme_data_release (in);
 
   err = gpgme_data_new_from_file (&in, seckey_1_asc, 1);
+  free (seckey_1_asc);
   fail_if_err (err);
 
   err = gpgme_op_import (ctx, in);
@@ -237,5 +251,24 @@ main (int argc, char *argv[])
   gpgme_data_release (in);
 
   gpgme_release (ctx);
+
+  err = gpgme_new (&ctx);
+  fail_if_err (err);
+
+  err = gpgme_get_key (ctx, "0xADAB7FCC1F4DE2616ECFA402AF82244F9CD9FD55", &key, 0);
+  fail_if_err (err);
+
+  if (!key)
+    {
+      fprintf (stderr, "Imported key not found\n");
+      exit (1);
+    }
+  if (key->origin != GPGME_KEYORG_WKD)
+    {
+      fprintf (stderr, "Key has unexpected origin: %d\n", key->origin);
+      exit (1);
+    }
+
+
   return 0;
 }
