@@ -1,21 +1,22 @@
 /* run-verify.c  - Helper to perform a verify operation
-   Copyright (C) 2009 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (C) 2009 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 /* We need to include config.h so that we know whether we are building
    with large file system (LFS) support. */
@@ -136,10 +137,11 @@ print_result (gpgme_verify_result_t result)
   gpgme_tofu_info_t ti;
   int count = 0;
 
-  printf ("Original file name: %s\n", nonnull(result->file_name));
+  printf ("Original file name .: %s\n", nonnull(result->file_name));
+  printf ("MIME flag ..........: %s\n", result->is_mime? "yes":"no");
   for (sig = result->signatures; sig; sig = sig->next)
     {
-      printf ("Signature %d\n", count++);
+      printf ("Signature ...: %d\n", count++);
       printf ("  status ....: %s\n", gpgme_strerror (sig->status));
       printf ("  summary ...:"); print_summary (sig->summary); putchar ('\n');
       printf ("  fingerprint: %s\n", nonnull (sig->fpr));
@@ -157,23 +159,31 @@ print_result (gpgme_verify_result_t result)
               sig->pka_trust == 0? "n/a" :
               sig->pka_trust == 1? "bad" :
               sig->pka_trust == 2? "okay": "RFU");
-      printf ("  other flags:%s%s\n",
+      printf ("  other flags:%s%s%s\n",
               sig->wrong_key_usage? " wrong-key-usage":"",
-              sig->chain_model? " chain-model":""
+              sig->chain_model? " chain-model":"",
+              sig->is_de_vs? " de-vs":""
               );
       for (nt = sig->notations; nt; nt = nt->next)
         {
-          printf ("  notation ..: '%s'\n", nt->name);
-          if (strlen (nt->name) != nt->name_len)
-            printf ("    warning : name larger (%d)\n", nt->name_len);
-          printf ("    flags ...:%s%s (0x%02x)\n",
-                  nt->critical? " critical":"",
-                  nt->human_readable? " human":"",
-                  nt->flags);
-          if (nt->value)
-            printf ("    value ...: '%s'\n", nt->value);
+          if (nt->name)
+            {
+              printf ("  notation ..: '%s'\n", nt->name);
+              if (strlen (nt->name) != nt->name_len)
+                printf ("    warning .: name larger (%d)\n", nt->name_len);
+              printf ("    flags ...:%s%s (0x%02x)\n",
+                      nt->critical? " critical":"",
+                      nt->human_readable? " human":"",
+                      nt->flags);
+              if (nt->value)
+                printf ("    value ...: '%s'\n", nt->value);
+            }
+          else
+            {
+              printf ("  policy ....: '%s'\n", nt->value);
+            }
           if ((nt->value?strlen (nt->value):0) != nt->value_len)
-            printf ("    warning : value larger (%d)\n", nt->value_len);
+            printf ("    warning .: value larger (%d)\n", nt->value_len);
         }
       if (sig->key)
         {
@@ -221,6 +231,10 @@ show_usage (int ex)
          "  --status         print status lines from the backend\n"
          "  --openpgp        use the OpenPGP protocol (default)\n"
          "  --cms            use the CMS protocol\n"
+         "  --sender MBOX    use MBOX as sender address\n"
+         "  --repeat N       repeat the operation N times\n"
+         "  --auto-key-retrieve\n"
+         "  --auto-key-import\n"
          , stderr);
   exit (ex);
 }
@@ -230,15 +244,14 @@ int
 main (int argc, char **argv)
 {
   int last_argc = -1;
-  gpgme_error_t err;
-  gpgme_ctx_t ctx;
+  const char *s;
   gpgme_protocol_t protocol = GPGME_PROTOCOL_OpenPGP;
-  FILE *fp_sig = NULL;
-  gpgme_data_t sig = NULL;
-  FILE *fp_msg = NULL;
-  gpgme_data_t msg = NULL;
-  gpgme_verify_result_t result;
   int print_status = 0;
+  const char *sender = NULL;
+  int auto_key_retrieve = 0;
+  int auto_key_import = 0;
+  int repeats = 1;
+  int i;
 
   if (argc)
     { argc--; argv++; }
@@ -273,6 +286,32 @@ main (int argc, char **argv)
           protocol = GPGME_PROTOCOL_CMS;
           argc--; argv++;
         }
+      else if (!strcmp (*argv, "--sender"))
+        {
+          argc--; argv++;
+          if (!argc)
+            show_usage (1);
+          sender = *argv;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--repeat"))
+        {
+            argc--; argv++;
+            if (!argc)
+                show_usage (1);
+            repeats = atoi (*argv);
+            argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--auto-key-retrieve"))
+        {
+          auto_key_retrieve = 1;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--auto-key-import"))
+        {
+          auto_key_import = 1;
+          argc--; argv++;
+        }
       else if (!strncmp (*argv, "--", 2))
         show_usage (1);
 
@@ -281,69 +320,120 @@ main (int argc, char **argv)
   if (argc < 1 || argc > 2)
     show_usage (1);
 
-  fp_sig = fopen (argv[0], "rb");
-  if (!fp_sig)
+  init_gpgme (protocol);
+
+  for (i = 0; i < repeats; i++)
     {
-      err = gpgme_error_from_syserror ();
-      fprintf (stderr, PGM ": can't open `%s': %s\n",
-               argv[0], gpgme_strerror (err));
-      exit (1);
-    }
-  if (argc > 1)
-    {
-      fp_msg = fopen (argv[1], "rb");
-      if (!fp_msg)
+      gpgme_error_t err;
+      gpgme_ctx_t ctx;
+      FILE *fp_sig = NULL;
+      gpgme_data_t sig = NULL;
+      FILE *fp_msg = NULL;
+      gpgme_data_t msg = NULL;
+      gpgme_verify_result_t result;
+
+      if (repeats > 1)
+        {
+          printf ("Repeat: %i\n", i);
+        }
+
+      fp_sig = fopen (argv[0], "rb");
+      if (!fp_sig)
         {
           err = gpgme_error_from_syserror ();
           fprintf (stderr, PGM ": can't open `%s': %s\n",
-                   argv[1], gpgme_strerror (err));
+                   argv[0], gpgme_strerror (err));
           exit (1);
         }
-    }
+      if (argc > 1)
+        {
+          fp_msg = fopen (argv[1], "rb");
+          if (!fp_msg)
+            {
+              err = gpgme_error_from_syserror ();
+              fprintf (stderr, PGM ": can't open `%s': %s\n",
+                       argv[1], gpgme_strerror (err));
+              exit (1);
+            }
+        }
 
-  init_gpgme (protocol);
+      err = gpgme_new (&ctx);
+      fail_if_err (err);
+      gpgme_set_protocol (ctx, protocol);
+      if (print_status)
+        {
+          gpgme_set_status_cb (ctx, status_cb, NULL);
+          gpgme_set_ctx_flag (ctx, "full-status", "1");
+        }
+      /* gpgme_set_ctx_flag (ctx, "raw-description", "1"); */
 
-  err = gpgme_new (&ctx);
-  fail_if_err (err);
-  gpgme_set_protocol (ctx, protocol);
-  if (print_status)
-    {
-      gpgme_set_status_cb (ctx, status_cb, NULL);
-      gpgme_set_ctx_flag (ctx, "full-status", "1");
-    }
-  /* gpgme_set_ctx_flag (ctx, "raw-description", "1"); */
+      if (auto_key_retrieve)
+        {
+          gpgme_set_ctx_flag (ctx, "auto-key-retrieve", "1");
+          s = gpgme_get_ctx_flag (ctx, "auto-key-retrieve");
+          if (!s || strcmp (s, "1"))
+            {
+              fprintf (stderr, PGM ": gpgme_get_ctx_flag failed for '%s'\n",
+                       "auto-key-retrieve");
+              exit (1);
+            }
+        }
 
-  err = gpgme_data_new_from_stream (&sig, fp_sig);
-  if (err)
-    {
-      fprintf (stderr, PGM ": error allocating data object: %s\n",
-               gpgme_strerror (err));
-      exit (1);
-    }
-  if (fp_msg)
-    {
-      err = gpgme_data_new_from_stream (&msg, fp_msg);
+      if (auto_key_import)
+        {
+          gpgme_set_ctx_flag (ctx, "auto-key-import", "1");
+          s = gpgme_get_ctx_flag (ctx, "auto-key-import");
+          if (!s || strcmp (s, "1"))
+            {
+              fprintf (stderr, PGM ": gpgme_get_ctx_flag failed for '%s'\n",
+                       "auto-key-import");
+              exit (1);
+            }
+        }
+
+      if (sender)
+        {
+          err = gpgme_set_sender (ctx, sender);
+          fail_if_err (err);
+        }
+
+      err = gpgme_data_new_from_stream (&sig, fp_sig);
       if (err)
         {
           fprintf (stderr, PGM ": error allocating data object: %s\n",
                    gpgme_strerror (err));
           exit (1);
         }
+      if (fp_msg)
+        {
+          err = gpgme_data_new_from_stream (&msg, fp_msg);
+          if (err)
+            {
+              fprintf (stderr, PGM ": error allocating data object: %s\n",
+                       gpgme_strerror (err));
+              exit (1);
+            }
+        }
+
+      err = gpgme_op_verify (ctx, sig, msg, NULL);
+      result = gpgme_op_verify_result (ctx);
+      if (result)
+        print_result (result);
+      if (err)
+        {
+          fprintf (stderr, PGM ": verify failed: %s\n", gpgme_strerror (err));
+          exit (1);
+        }
+
+      gpgme_data_release (msg);
+      gpgme_data_release (sig);
+
+      gpgme_release (ctx);
+
+      if (fp_msg)
+        fclose (fp_msg);
+      if (fp_sig)
+        fclose (fp_sig);
     }
-
-  err = gpgme_op_verify (ctx, sig, msg, NULL);
-  result = gpgme_op_verify_result (ctx);
-  if (result)
-    print_result (result);
-  if (err)
-    {
-      fprintf (stderr, PGM ": verify failed: %s\n", gpgme_strerror (err));
-      exit (1);
-    }
-
-  gpgme_data_release (msg);
-  gpgme_data_release (sig);
-
-  gpgme_release (ctx);
   return 0;
 }
